=== modified file 'drizzled/my_hash.cc'
--- drizzled/my_hash.cc	2010-02-04 08:14:46 +0000
+++ drizzled/my_hash.cc	2010-06-16 17:29:00 +0000
@@ -122,7 +122,6 @@
   hash_free_elements(hash);
   hash->free= 0;
   delete_dynamic(&hash->array);
-  return;
 }
 
 /* some helper functions */

=== modified file 'drizzled/sql_base.cc'
--- drizzled/sql_base.cc	2010-06-15 13:37:26 +0000
+++ drizzled/sql_base.cc	2010-06-16 17:29:00 +0000
@@ -86,8 +86,8 @@
                                bool )
 {
   Table *entry=(Table*) record;
-  *length= entry->getShare()->getCacheKeySize();
-  return (unsigned char*) entry->getMutableShare()->getCacheKey();
+  *length= entry->getShare()->getCacheKey().size();
+  return (unsigned char*) &entry->getShare()->getCacheKey()[0];
 }
 
 HASH *get_open_cache()
@@ -142,8 +142,8 @@
 void close_handle_and_leave_table_as_lock(Table *table)
 {
   TableShare *share, *old_share= table->getMutableShare();
-
   assert(table->db_stat);
+  assert(table->getShare()->getType() == message::Table::STANDARD);
 
   /*
     Make a local copy of the table share and free the current one.
@@ -151,9 +151,10 @@
     the table defintion cache as soon as the last instance is removed
   */
   TableIdentifier identifier(table->getShare()->getSchemaName(), table->getShare()->getTableName(), message::Table::INTERNAL);
+  const TableIdentifier::Key &key(identifier.getKey());
   share= new TableShare(identifier.getType(),
                         identifier,
-                        const_cast<char *>(old_share->getCacheKey()),  static_cast<uint32_t>(old_share->getCacheKeySize()));
+                        const_cast<char *>(&key[0]),  static_cast<uint32_t>(old_share->getCacheKeySize()));
 
   table->cursor->close();
   table->db_stat= 0;                            // Mark cursor closed
@@ -687,8 +688,9 @@
 
   for (Table *table= temporary_tables ; table ; table= table->getNext())
   {
-    if (table->getShare()->getCacheKeySize() == key_length &&
-        not memcmp(table->getMutableShare()->getCacheKey(), key, key_length))
+    const TableIdentifier::Key &share_key(table->getShare()->getCacheKey());
+    if (share_key.size() == key_length &&
+        not memcmp(&share_key[0], key, key_length))
     {
       return table;
     }
@@ -703,13 +705,9 @@
 
 Table *Session::find_temporary_table(TableIdentifier &identifier)
 {
-  const TableIdentifier::Key &key(identifier.getKey());
-
   for (Table *table= temporary_tables ; table ; table= table->getNext())
   {
-    if (table->getShare()->getCacheKeySize() == key.size() &&
-        not memcmp(table->getMutableShare()->getCacheKey(), &key[0], key.size()))
-
+    if (identifier.getKey() == table->getShare()->getCacheKey())
       return table;
   }
 
@@ -793,10 +791,9 @@
   char key[MAX_DBKEY_LENGTH];
   uint32_t key_length= find->getShare()->getCacheKeySize();
   Table *list, **prev;
-
   safe_mutex_assert_owner(&LOCK_open);
 
-  memcpy(key, find->getMutableShare()->getCacheKey(), key_length);
+  memcpy(key, &find->getMutableShare()->getCacheKey()[0], key_length);
   /*
     Note that we need to hold LOCK_open while changing the
     open_tables list. Another thread may work on it.
@@ -809,7 +806,7 @@
     list= *prev;
 
     if (list->getShare()->getCacheKeySize() == key_length &&
-        not memcmp(list->getMutableShare()->getCacheKey(), key, key_length))
+        not memcmp(&list->getShare()->getCacheKey()[0], key, key_length))
     {
       /* Remove table from open_tables list. */
       *prev= list->getNext();
@@ -1158,7 +1155,7 @@
   */
   for (table= temporary_tables; table ; table=table->getNext())
   {
-    if (table->getShare()->getCacheKeySize() == key_length && !memcmp(table->getMutableShare()->getCacheKey(), &key[0], key_length))
+    if (table->getShare()->getCacheKey() == key)
     {
       /*
         We're trying to use the same temporary table twice in a query.
@@ -1776,15 +1773,14 @@
 {
   do
   {
-    char *key= const_cast<char *>(table->getMutableShare()->getCacheKey());
-    uint32_t key_length= table->getShare()->getCacheKeySize();
+    const TableIdentifier::Key &key(table->getShare()->getCacheKey());
 
     HASH_SEARCH_STATE state;
-    for (Table *search= (Table*) hash_first(&open_cache, (unsigned char*) key,
-                                            key_length, &state);
+    for (Table *search= (Table*) hash_first(&open_cache, (unsigned char*) &key[0],
+                                            key.size(), &state);
          search ;
-         search= (Table*) hash_next(&open_cache, (unsigned char*) key,
-                                    key_length, &state))
+         search= (Table*) hash_next(&open_cache, (unsigned char*) &key[0],
+                                    key.size(), &state))
     {
       if (search->in_use == table->in_use)
         continue;                               // Name locked by this thread

=== modified file 'drizzled/table_share.cc'
--- drizzled/table_share.cc	2010-06-15 13:37:26 +0000
+++ drizzled/table_share.cc	2010-06-16 17:29:00 +0000
@@ -591,8 +591,6 @@
   assert(identifier.getKey() == key);
 
   table_charset= 0;
-  memset(&db, 0, sizeof(LEX_STRING));
-  memset(&table_name, 0, sizeof(LEX_STRING));
   memset(&path, 0, sizeof(LEX_STRING));
   memset(&normalized_path, 0, sizeof(LEX_STRING));
 
@@ -825,6 +823,37 @@
   path.length= normalized_path.length= strlen(new_path);
 }
 
+TableShare::~TableShare() 
+{
+  assert(ref_count == 0);
+
+  /*
+    If someone is waiting for this to be deleted, inform it about this.
+    Don't do a delete until we know that no one is refering to this anymore.
+  */
+  if (tmp_table == message::Table::STANDARD)
+  {
+    /* share->mutex is locked in release_table_share() */
+    while (waiting_on_cond)
+    {
+      pthread_cond_broadcast(&cond);
+      pthread_cond_wait(&cond, &mutex);
+    }
+    /* No thread refers to this anymore */
+    pthread_mutex_unlock(&mutex);
+    pthread_mutex_destroy(&mutex);
+    pthread_cond_destroy(&cond);
+  }
+  hash_free(&name_hash);
+
+  storage_engine= NULL;
+
+  delete table_proto;
+  table_proto= NULL;
+
+  mem_root.free_root(MYF(0));                 // Free's share
+}
+
 void TableShare::setIdentifier(TableIdentifier &identifier_arg)
 {
   private_key_for_cache.clear();

=== modified file 'drizzled/table_share.h'
--- drizzled/table_share.h	2010-06-15 13:37:26 +0000
+++ drizzled/table_share.h	2010-06-16 17:29:00 +0000
@@ -66,36 +66,7 @@
              TableIdentifier &identifier,
              char *path_arg= NULL, uint32_t path_length_arg= 0); // Shares for cache
 
-  ~TableShare() 
-  {
-    assert(ref_count == 0);
-
-    /*
-      If someone is waiting for this to be deleted, inform it about this.
-      Don't do a delete until we know that no one is refering to this anymore.
-    */
-    if (tmp_table == message::Table::STANDARD)
-    {
-      /* share->mutex is locked in release_table_share() */
-      while (waiting_on_cond)
-      {
-        pthread_cond_broadcast(&cond);
-        pthread_cond_wait(&cond, &mutex);
-      }
-      /* No thread refers to this anymore */
-      pthread_mutex_unlock(&mutex);
-      pthread_mutex_destroy(&mutex);
-      pthread_cond_destroy(&cond);
-    }
-    hash_free(&name_hash);
-
-    storage_engine= NULL;
-
-    delete table_proto;
-    table_proto= NULL;
-
-    mem_root.free_root(MYF(0));                 // Free's share
-  };
+  ~TableShare();
 
 private:
   /** Category of this table. */
@@ -290,10 +261,10 @@
     return path.str;
   }
 
-  const char *getCacheKey() const // This should never be called when we aren't looking at a cache.
+  const TableIdentifier::Key& getCacheKey() const // This should never be called when we aren't looking at a cache.
   {
     assert(private_key_for_cache.size());
-    return &private_key_for_cache[0];
+    return private_key_for_cache;
   }
 
   size_t getCacheKeySize() const

